//+------------------------------------------------------------------+
//|                                                net_wprExpert.mq5 |
//|                                                         Tapochun |
//|                         https://login.mql5.com/ru/users/tapochun |
//+------------------------------------------------------------------+
#property copyright "Tapochun"
#property link      "https://login.mql5.com/ru/users/tapochun"
#property version   "1.00"
//+------------------------------------------------------------------+
//|  																|
//+------------------------------------------------------------------+
input int inpPeriod = 6;				//   WPR
input double inpVol = 0.01;			//  
input double inpUpBorder = -5.0;		//     
input double inpDnBorder = -95.0;	//     
input ulong inpMagic = 777;			//   
//+------------------------------------------------------------------+
//|  															|
//+------------------------------------------------------------------+
int globWprHandle;								//   WPR
const string expertComment = "wprExpert";	//  

struct DATA_POSITION								//     
	{
	 int id;											//  
	 ENUM_POSITION_TYPE type;					//  
	 double nearMainLevel;						//    (    )
	};
DATA_POSITION globPosition;					//     
int globFactor;									//  (      )
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
   {
    if( _Digits < 4 )								//    
    	{
    	 Print( __FUNCTION__,": !     ,  !" );
    	 return( INIT_PARAMETERS_INCORRECT );	//   
    	}
    //   
    if( _Digits == 5 ) globFactor = 1000;		//    
    if( _Digits == 4 ) globFactor = 100;		//    
    
	 globWprHandle = iWPR( _Symbol, _Period, inpPeriod );	//    WPR
	 if( globWprHandle == INVALID_HANDLE )						//    
	 	{
	 	 Print( __FUNCTION__,":  #",GetLastError(),".   WPR  !" );
	 	 return( INIT_FAILED );										//   
	 	}
    return(INIT_SUCCEEDED);
   }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
   {
	 
   }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
   {
    //    
    NetModule( globPosition, globFactor );
    
	 static datetime savedTime;						//     
	 if( !CheckNewCandle( savedTime ) ) return;	//      - 
	 
	 double wprValue[ 2 ];								//   WPR   
	 int num = CopyBuffer( globWprHandle, 0, 1, 2, wprValue );	//       
	 if( num == -1 ) 															//    
	 	{
	 	 Print( __FUNCTION__,":  #",GetLastError(),".     ! num = ",num );
	 	 return;																	// 
	 	}
	 
	 //Print( __FUNCTION__,": WPR[ 1 ]: ",DoubleToString( wprValue[ 0 ], 1 ),", WPR[ 2 ]: ",DoubleToString( wprValue[ 1 ], 1 ) );
	 
	 bool posExist = PositionSelect( _Symbol );	//   
	 if( !posExist )										//    
	 	{
	 	 //   
	 	 if( wprValue[ 0 ] > inpUpBorder && wprValue[ 1 ] < inpUpBorder )			//        
	 	 	{													
	 	 	 double bid = SymbolInfoDouble( _Symbol, SYMBOL_BID );									//   
	 	 	 OpenPosition( ORDER_TYPE_SELL, _Symbol, inpVol, bid, expertComment, inpMagic );	//     
	 	 	 return;
	 	 	}
	 	 if( wprValue[ 0 ] < inpDnBorder && wprValue[ 1 ] > inpDnBorder )			//        
	 	 	{													
	 	 	 double ask = SymbolInfoDouble( _Symbol, SYMBOL_ASK );									//   ask
	 	 	 OpenPosition( ORDER_TYPE_BUY, _Symbol, inpVol, ask, expertComment, inpMagic );	//     
	 	 	 return;
	 	 	}
	 	}
	 else 													//   
	   {
	    // ,   ""
	    double posVol = PositionGetDouble( POSITION_VOLUME );	//   
	    ENUM_POSITION_TYPE posType = (ENUM_POSITION_TYPE)PositionGetInteger( POSITION_TYPE );
	    
	    //            
	    if( posType == POSITION_TYPE_SELL && wprValue[ 0 ] < inpDnBorder && wprValue[ 1 ] > inpDnBorder )	
	    	{
	    	 double ask = SymbolInfoDouble( _Symbol, SYMBOL_ASK );				//   ask
	    	 double vol = PositionGetDouble( POSITION_VOLUME );					//   
	    	 vol += inpVol;																	//   
	    	 OpenPosition( ORDER_TYPE_BUY, _Symbol, NormalizeDouble( vol, 2 ), ask, expertComment, inpMagic );
	    	 return;
	    	}
	    
	    //            
	    if( posType == POSITION_TYPE_BUY && wprValue[ 0 ] > inpUpBorder && wprValue[ 1 ] < inpUpBorder )	
	    	{
	    	 double bid = SymbolInfoDouble( _Symbol, SYMBOL_BID );				//   
	    	 double vol = PositionGetDouble( POSITION_VOLUME );					//   
	    	 vol += inpVol;																	//   
	    	 OpenPosition( ORDER_TYPE_SELL, _Symbol, NormalizeDouble( vol, 2 ), bid, expertComment, inpMagic );
	    	 return;	
	    	}
	   }
   }
//+------------------------------------------------------------------+
//|     											|
//+------------------------------------------------------------------+
bool CheckNewCandle( datetime &savedTime		//     pos (out)
						 )
	{
	 datetime time = (datetime)SeriesInfoInteger( _Symbol, _Period, SERIES_LASTBAR_DATE );	//    
	 if( savedTime == time ) return( false );	//     -  
	 savedTime = time;								//     
	 return( true );									//   
	}
//+------------------------------------------------------------------+
//|   																|
//+------------------------------------------------------------------+
void OpenPosition( const ENUM_ORDER_TYPE type, 		//  
						 const string symbol,				//  
						 const double vol,					// 
						 const double price,					//  
						 const string comment,				//   
						 const ulong magic					//  
					  )
	{
	 MqlTradeRequest request = { 0 };
	 MqlTradeResult result = { 0 };
	 
	 request.action = TRADE_ACTION_DEAL;
	 request.symbol = symbol;
	 request.volume = vol;
	 request.type = type;
	 request.type_filling = ORDER_FILLING_FOK;
	 request.price = price;
	 request.comment = comment;
	 request.magic = magic;
	 
	 bool answer = OrderSend( request, result );
	 if( !answer )
	 	 Print( __FUNCTION__,": !    .  # ",GetLastError(),", retcode = ",result.retcode );
	}
//+------------------------------------------------------------------+
//|        	|
//+------------------------------------------------------------------+
void NetModule( DATA_POSITION &position,								//   
					 const int factor											// 
				  )
	{
	 if( !PositionSelect( _Symbol ) ) return;							//     - 
	 
	 double posProfit = PositionGetDouble( POSITION_PROFIT );	//    
	 if( posProfit > 0 ) return;											//      - 
	 
	 //   
	 int posID = (int)PositionGetInteger( POSITION_IDENTIFIER );										//  
	 ENUM_POSITION_TYPE posType = (ENUM_POSITION_TYPE)PositionGetInteger( POSITION_TYPE );		//   
	 //Print( __FUNCTION__,": ID  = ",posID,",   = ",EnumToString( posType ) );
	 
	 //    
	 if( position.id != posID || position.type != posType )		//    ..
	 	{																			// ..   ,   
	 	 position.id = posID;				//   ID  (  )
	 	 position.type = posType;			//    
	 	 if( posType == POSITION_TYPE_SELL )	//    
	 	 	 position.nearMainLevel = 0.0;		//      
	 	 else 											//    
	 	 	 position.nearMainLevel = DBL_MAX;
	 	}
	 
	 // ,    
	 double bid = SymbolInfoDouble( _Symbol, SYMBOL_BID );		//   
	 double ask = SymbolInfoDouble( _Symbol, SYMBOL_ASK );		//   
	 double posPrice = PositionGetDouble( POSITION_PRICE_OPEN );//   
	 
	 CheckPosition( posType, factor, posPrice, position.nearMainLevel, bid, ask );		
	 
	 
	}
//+------------------------------------------------------------------+
//|    													|
//+------------------------------------------------------------------+
void CheckPosition( const ENUM_POSITION_TYPE posType,	//  
						  const int factor,						// 
						  const double posPrice,				//   
						  double &posNearLevel,					//    ,    (out)
						  const double bid,						//   
						  const double ask						//   
						)
	{
	 int mainLevel;
	 int nextLevel;
	 double nextDoubleLevel = 0.0;
	 
	 if( posType == POSITION_TYPE_BUY )		//   
	 	{
	 	 if( posNearLevel == DBL_MAX )				//    
	 	 	{											//    
	 	 	 //   
		 	 mainLevel = (int)( posPrice/(_Point*factor) );			//     
		 	 nextLevel = mainLevel*10+5;									//    ( )
		 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;		//  
		 	 
		 	 Print( __FUNCTION__,":   : ",mainLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", ask = ",DoubleToString( ask, _Digits ) );
		 	 if( nextDoubleLevel > ask )									//      -  
		 	 	{ 
		 	 	 nextDoubleLevel = mainLevel*( _Point*factor );		//      
		 	 	 Print( __FUNCTION__,":   : ",mainLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", ask = ",DoubleToString( ask, _Digits ) );
		 	 	}	 	 
	 	 	}
	 	 //if( posNearLevel > 0.0 )				//    
	 	 if( posNearLevel != DBL_MAX )
	 	 	{
	 	 	 mainLevel = (int)( posNearLevel/( _Point*factor ) );	//     
	 	 	 nextLevel = mainLevel*10-5;									//    ( )
		 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;		//  
		 	 
		 	 Print( __FUNCTION__,":   : ",mainLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", ask = ",DoubleToString( ask, _Digits ) );
		 	 if( nextDoubleLevel > ask )									//      -  
		 	 	{ 
		 	 	 nextLevel -= 5;												//   
		 	 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;	//    
		 	 	 Print( __FUNCTION__,":   : ",nextLevel,".    = ",DoubleToString( nextDoubleLevel, _Digits ),", ask = ",DoubleToString( ask, _Digits ) );
		 	 	}	 	
	 	 	}
	 	 // ,         
	 	 Print( __FUNCTION__,":       ,  ask  < ",DoubleToString( nextDoubleLevel+50*_Point, _Digits )," (",DoubleToString( ask, _Digits ),")",
	 	 							", posNearLevel = ",DoubleToString( posNearLevel, _Digits ) );
	 	 if( ask < ( nextDoubleLevel+50*_Point ) && posNearLevel > nextDoubleLevel )					//      
	 	 	{
	 	 	 Print( __FUNCTION__,":     "+DoubleToString( ask, _Digits ) );
	 	 	 posNearLevel = nextDoubleLevel;								//    
	 	 	 OpenPosition( ORDER_TYPE_BUY, _Symbol, inpVol, ask, expertComment, inpMagic );	// 
	 	 	}
	 	}
	 else 																		//   
	 	{
	 	 if( posNearLevel == 0.0 )											//    
	 	 //if( posNearLevel == DBL_MAX )
	 	 	{																		//    
		 	 mainLevel = (int)( posPrice/(_Point*factor) );			//     
		 	 nextLevel = mainLevel*10+5;									//    ( )
		 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;		//  
		 	 
		 	 Print( __FUNCTION__,":   : ",mainLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", bid = ",DoubleToString( bid, _Digits ) );
		 	 if( nextDoubleLevel < bid )											//      -  
		 	 	{ 
		 	 	 nextLevel += 5;												//    
		 	 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;	//    
		 	 	 Print( __FUNCTION__,":   : ",nextLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", bid = ",DoubleToString( bid, _Digits ) );
		 	 	}	 	 
	 	 	}
	 	 if( posNearLevel > 0.0 )											//    
	 	 //if( posNearLevel != DBL_MAX )
	 	 	{
	 	 	 mainLevel = (int)( posNearLevel/( _Point*factor ) );	//     
	 	 	 nextLevel = mainLevel*10+5;									//    ( )
		 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;			//  
		 	 
		 	 Print( __FUNCTION__,":   : ",mainLevel,".   = ",DoubleToString( nextDoubleLevel, _Digits ),", bid = ",DoubleToString( bid, _Digits ) );
		 	 if( nextDoubleLevel < bid )									//      -  
		 	 	{ 
		 	 	 nextLevel += 5;												//   
		 	 	 nextDoubleLevel = nextLevel*( _Point*factor )/10;	//    
		 	 	 Print( __FUNCTION__,":   : ",nextLevel,".    = ",DoubleToString( nextDoubleLevel, _Digits ),", bid = ",DoubleToString( bid, _Digits ) );
		 	 	}	 	
	 	 	}
	 	 // ,         
	 	 Print( __FUNCTION__,":       ,  bid  > ",DoubleToString( nextDoubleLevel-50*_Point, _Digits )," (",DoubleToString( bid, _Digits ),
	 	 							"), posNearLevel = ",DoubleToString( posNearLevel, _Digits ) );
	 	 if( bid > ( nextDoubleLevel-50*_Point ) && posNearLevel < nextDoubleLevel )					//      
	 	 	{
	 	 	 Print( __FUNCTION__,":     "+DoubleToString( bid, _Digits ) );
	 	 	 posNearLevel = nextDoubleLevel;								//    
	 	 	 OpenPosition( ORDER_TYPE_SELL, _Symbol, inpVol, bid, expertComment, inpMagic );	// 
	 	 	}
	 	}
	}